module image_moveouts

	use constants

	implicit none

	contains

subroutine flat_wb_cigs_moveout(nmd,omd,dmd,ng,og,dg,nhi,ohi,dhi,&
	& zwb,rho,zih,zig,zimh,zimg,hi,g,mzih,mzig)

	!! subroutine arguments
	integer,			intent(in)	:: nmd
	real,				intent(in)	:: omd
	real,				intent(in)	:: dmd
	integer,			intent(in)	:: ng
	real,				intent(in)	:: og
	real,				intent(in)	:: dg
	integer,			intent(in)	:: nhi
	real,				intent(in)	:: ohi
	real,				intent(in)	:: dhi
	real,				intent(in)	:: zwb
	real,				intent(in)	:: rho
	real,dimension(nhi),intent(out)	:: zih
	real,dimension(ng),	intent(out)	:: zig
	real,dimension(nmd),intent(out)	:: zimh
	real,dimension(nmd),intent(out)	:: zimg
	real,dimension(nhi),intent(out)	:: hi
	real,dimension(ng),	intent(out)	:: g
	real,dimension(nmd),intent(out)	:: mzih
	real,dimension(nmd),intent(out)	:: mzig
	
	!! internal variables
	integer :: ig,ihi,imd
	real	:: rho2, hrho

	! set auxiliary variables
	rho2=rho**2
	hrho=rho/2.

	! loop over subsurface half-offsets
	do ihi=1,nhi
		hi(ihi)=ohi+(ihi-1)*dhi
		zih(ihi)=zwb+hrho*sqrt(4*zwb**2+(2*hi(ihi))**2/(1-rho2))
	end do

	! loop over aperture angles
	do ig=1,ng
		g(ig)=og+(ig-1)*dg
		zig(ig)=zwb*(1+(cos(g(ig))*(rho2-tan(g(ig))**2*(1-rho2)))/ &
			& (sqrt(rho2-sin(g(ig))**2))) 
!write(0,*)'ig=',ig,'pi=',pi,'g=',g(ig),'zig=',zig(ig)
	end do

	! create image section for a given offset
	!!!!! OJOOOOOOOOOOOOOO: WRONG !!!!!!!!!!!!!!!!!!!!!
	do imd=1,nmd
		mzih(imd)=omd+(imd-1)*dmd
		zimh(imd)=zwb
	end do
	
	! create image section for a given angle
	!!!!! OJOOOOOOOOOOOOOO: WRONG !!!!!!!!!!!!!!!!!!!!!
	do imd=1,nmd
		mzig(imd)=omd+(imd-1)*dmd
		zimg(imd)=zwb
	end do
	
	return
end subroutine flat_wb_cigs_moveout

subroutine flat_diffmul_cigs_moveout(xmzi,xhi,xg,nmd,omd,dmd,nhd,ohd,dhd, &
	& xdiff,zwb,rho,v1,zih,zig,zimh,zimg,nhi,dhi,hi,ng,dg,g,mzih,mzig, &
	& count_hi,count_g,count_mzih,count_mzig)

	!! subroutine arguments
	real,				intent(in)	:: xmzi
	real,				intent(in)	:: xhi
	real,				intent(in)	:: xg
	integer,			intent(in)	:: nmd
	real,				intent(in)	:: omd
	real,				intent(in)	:: dmd
	integer,			intent(in)	:: nhd
	real,				intent(in)	:: ohd
	real,				intent(in)	:: dhd
	real,				intent(in)	:: xdiff
	real,				intent(in)	:: zwb
	real,				intent(in)	:: rho
	real,				intent(in)	:: v1
	integer,			intent(in)	:: nhi
	real,				intent(in)	:: dhi
	integer,			intent(in)	:: ng
	real,				intent(in)	:: dg
	real,dimension(nhi),intent(out)	:: zih
	real,dimension(ng),	intent(out)	:: zig
	real,dimension(nmd),intent(out)	:: zimh
	real,dimension(nmd),intent(out)	:: zimg
	real,dimension(nhi),intent(out)	:: hi
	real,dimension(ng),	intent(out)	:: g
	real,dimension(nmd),intent(out)	:: mzih
	real,dimension(nmd),intent(out)	:: mzig
	integer,			intent(out) :: count_hi
	integer,			intent(out) :: count_g
	integer,			intent(out) :: count_mzih
	integer,			intent(out) :: count_mzig
	
	!! internal variables
	integer :: ihd,imd
	real	:: md, hd
	real	:: s_alpha_s, s_alpha_r, c_alpha_s, c_alpha_r
	real	:: s_beta_s, s_beta_r, c_beta_s, c_beta_r
	real	:: ts1, tr1, ts2, tr2
	real	:: hzi, zzi, mzi, zzig
	real	:: alpha_s, alpha_r, beta_s, beta_r, gamma
	real	:: temp1, temp2, zwb2, rho2 

!write(0,*)'Before counters'
	! initialize counters
    count_hi=0;	count_g=0; count_mzih=0; count_mzig=0
	zwb2=zwb**2
	rho2=rho**2

write(0,*)'ohd=',ohd,'dhd=',dhd,'nhd=',nhd,'xhi=',xhi,'dhi=',dhi/2

	! loop over data space cmps
    do imd=1,nmd

        md=omd+(imd-1)*dmd

		! loop over data space surface offsets
        do ihd=1,nhd

            hd=ohd+(ihd-1)*dhd;

            ! compute the sines and cosines of the take-off angles 
			! at the sources and receivers
            s_alpha_s=(xdiff-(md-hd))/(3*sqrt(((xdiff-(md-hd))/3.)**2+zwb2))
            s_alpha_r=(((md+hd)-xdiff)/sqrt(((md+hd)-xdiff)**2+zwb2))
            c_alpha_s=zwb/sqrt(((xdiff-(md-hd))/3.)**2+zwb2)
            c_alpha_r=zwb/sqrt(((md+hd)-xdiff)**2+zwb2)

!write(0,*)'md=',md,'hd=',hd,'s_alpha_s=',s_alpha_s,'s_alpha_r=',s_alpha_r,'c_alpha_s=',c_alpha_s,'c_alpha_r=',c_alpha_r
            ! compute cosines of refracted source and receiver rays
            temp1=1-rho2*(s_alpha_s)**2
            s_beta_s=rho*s_alpha_s
            s_beta_r=rho*s_alpha_r
            if (temp1<0) then
				cycle
            else
                c_beta_s=sqrt(temp1);
            end if
            temp2=1-rho2*(s_alpha_r)**2
            if (temp2<0) then
                cycle;
            else
                c_beta_r=sqrt(temp2)
            end if
!write(0,*)'imd=',imd,'md=',md,'ihd=',ihd,'hd=',hd,'c_beta_s=',c_beta_s,'c_beta_r=',c_beta_r

            ! compute the non-refracted traveltime segments
            if (s_alpha_s==0) then
                ts1=zwb/v1
            else
                ts1=abs((xdiff-(md-hd))/(3*v1*s_alpha_s))
            end if
            if (s_alpha_r==0) then
                tr1=zwb/v1
            else
                tr1=abs(((md+hd)-xdiff)/(v1*s_alpha_r))
            end if
            ts2=ts1; tr2=ts1;

            ! compute the image space coordinates
			call compute_image_coords(hd,v1,ts1,s_alpha_s,tr1,ts2,tr2,s_alpha_r,&
				& rho,c_alpha_r,c_alpha_s,s_beta_s,s_beta_r,hzi,zwb,c_beta_r,&
				& c_beta_s,zzi,md,mzi,gamma,zzig)
			if (gamma==-9999.) cycle

!write(0,*)'imd=',imd,'ihd=',ihd,'hzi=',hzi,'zzi=',zzi,'mzi=',mzi
			call set_output(mzi,xmzi,dmd,count_hi,hi,hzi,zih,zzi,nhi,xhi,dhi,&
				& count_mzih,zimh,mzih,nmd,count_g,g,gamma,zig,zzig,ng,xg,dg,&
				& count_mzig,mzig,zimg)
        end do
    end do
write(0,*)'BEFORE RETURN'
	return
end subroutine flat_diffmul_cigs_moveout

subroutine dip_wb_cigs_moveout(xmzi,xhi,xg,nmd,omd,dmd,nhd,ohd,dhd, &
	& dip,rho,v1,zih,zig,zimh,zimg,nhi,dhi,hi,ng,dg,g,mzih,mzig, &
	& count_hi,count_g,count_mzih,count_mzig)

	!! subroutine arguments
	real,				intent(in)	:: xmzi
	real,				intent(in)	:: xhi
	real,				intent(in)	:: xg
	integer,			intent(in)	:: nmd
	real,				intent(in)	:: omd
	real,				intent(in)	:: dmd
	integer,			intent(in)	:: nhd
	real,				intent(in)	:: ohd
	real,				intent(in)	:: dhd
	real,				intent(in)	:: dip
	real,				intent(in)	:: rho
	real,				intent(in)	:: v1
	integer,			intent(in)	:: nhi
	real,				intent(in)	:: dhi
	integer,			intent(in)	:: ng
	real,				intent(in)	:: dg
	real,dimension(nhi),intent(out)	:: zih
	real,dimension(ng),	intent(out)	:: zig
	real,dimension(nmd),intent(out)	:: zimh
	real,dimension(nmd),intent(out)	:: zimg
	real,dimension(nhi),intent(out)	:: hi
	real,dimension(ng),	intent(out)	:: g
	real,dimension(nmd),intent(out)	:: mzih
	real,dimension(nmd),intent(out)	:: mzig
	integer,			intent(out) :: count_hi
	integer,			intent(out) :: count_g
	integer,			intent(out) :: count_mzih
	integer,			intent(out) :: count_mzig
	
	!! internal variables
	integer :: ihd,imd
	real	:: md, hd
	real	:: s_alpha_s, s_alpha_r, c_alpha_s, c_alpha_r
	real	:: s_beta_s, s_beta_r, c_beta_s, c_beta_r
	real	:: ts1, tr1, ts2, tr2
	real	:: hzi, zzi, mzi, zzig
	real	:: alpha_s, alpha_r, beta_s, beta_r, gamma
	real	:: temp1, temp2, zwb, zwb2, rho2, vnmo 
	real	:: hatZD, tildeZD
	real	:: tm, tm0

	! initialize counters
    count_hi=0;	count_g=0; count_mzih=0; count_mzig=0
    vnmo=v1/cos(2*dip);
	rho2=rho**2

write(0,*)'AFTER INITIALIZATIOn'
write(0,*)'ohd=',ohd,'dhd=',dhd
write(0,*)'xmzi=',xmzi,'dmd=',dmd,'dhi=',dhi,'dg=',dg
	! loop over data space cmps
    do imd=1,nmd

        md=omd+(imd-1)*dmd
        zwb=md*tan(dip);
        zwb2=zwb*zwb;
       	hatZD=md*sin(2*dip);
       	tm0=2*hatZD/v1;

		! loop over data space surface offsets
        do ihd=1,nhd

            hd=ohd+(ihd-1)*dhd;

!write(0,*)'md=',md,'hd=',hd
			tm=sqrt(tm0**2+(2*hd/vnmo)**2);
            alpha_s=asin(2*hd*cos(2*dip)/(v1*tm))-2*dip;
            alpha_r=alpha_s+4*dip;

!write(0,*)'md=',md,'hd=',hd,'tm=',tm
!write(0,*)'alpha_s=',alpha_s,'alpha_r=',alpha_r
            ! compute tildeZD, the perpendicular distance to 
			! the reflector at the CMP
            tildeZD=(v1*tm0*cos(dip))/(2*(1+cos(2*dip)));

            ! compute the sines and cosines of the take-off angles 
			! at the sources and receivers
            s_alpha_s=sin(alpha_s);
            c_alpha_s=cos(alpha_s);
            s_alpha_r=sin(alpha_r);
            c_alpha_r=cos(alpha_r);
            temp1=1-rho2*(sin(alpha_s+dip))**2;
            if (temp1<0) then
                cycle;
            end if
            s_beta_s=rho*sin(alpha_s+dip)*cos(dip)-sqrt(temp1)*sin(dip);
            c_beta_s=sqrt(temp1)*cos(dip)+rho*sin(alpha_s+dip)*sin(dip);
            temp2=1-rho2*(sin(alpha_s+3*dip))**2;
            if (temp2<0) then
                cycle;
            end if
            s_beta_r=rho*sin(alpha_s+3*dip)*cos(dip)+sqrt(temp2)*sin(dip);
            c_beta_r=sqrt(temp2)*cos(dip)-rho*sin(alpha_s+3*dip)*sin(dip);

            ! compute the individual traveltime segments
            ts1=(tildeZD-hd*sin(dip))/(v1*cos(dip+alpha_s));
            ts2=(ts1*c_alpha_s)/cos(alpha_s+2*dip);
            tr2=(ts2*cos(alpha_s+dip))/cos(alpha_s+3*dip);
            tr1=(tr2*cos(alpha_s+2*dip))/cos(alpha_s+4*dip);

            !beta_s=asin(s_beta_s);
            !beta_r=asin(s_beta_r);
            !tts2=(tr1*c_alpha_r-ts1*c_alpha_s+rho*(ts2+tr2)*c_beta_r)/&
			!	& (rho*(c_beta_s+c_beta_r));
            !ttr2=(ts1*c_alpha_s-tr1*c_alpha_r+rho*(ts2+tr2)*c_beta_s)/&
			!	& (rho*(c_beta_s+c_beta_r));

            ! compute the image space coordinates
			call compute_image_coords(hd,v1,ts1,s_alpha_s,tr1,ts2,tr2,s_alpha_r,&
				& rho,c_alpha_r,c_alpha_s,s_beta_s,s_beta_r,hzi,zwb,c_beta_r,&
				& c_beta_s,zzi,md,mzi,gamma,zzig)
			if (gamma==-9999.) cycle

			! set output arrays
			call set_output(mzi,xmzi,dmd,count_hi,hi,hzi,zih,zzi,nhi,xhi,dhi,&
				& count_mzih,zimh,mzih,nmd,count_g,g,gamma,zig,zzig,ng,xg,dg,&
				& count_mzig,mzig,zimg)
        end do
    end do
	return
end subroutine dip_wb_cigs_moveout

subroutine dip_diffmul_cigs_moveout(xmzi,xhi,xg,nmd,omd,dmd,nhd,ohd,dhd, &
	& xdiff,dip,rho,v1,zih,zig,zimh,zimg,nhi,dhi,hi,ng,dg,g,mzih,mzig, &
	& count_hi,count_g,count_mzih,count_mzig)

	!! subroutine arguments
	real,				intent(in)	:: xmzi
	real,				intent(in)	:: xhi
	real,				intent(in)	:: xg
	integer,			intent(in)	:: nmd
	real,				intent(in)	:: omd
	real,				intent(in)	:: dmd
	integer,			intent(in)	:: nhd
	real,				intent(in)	:: ohd
	real,				intent(in)	:: dhd
	real,				intent(in)	:: dip
	real,				intent(in)	:: xdiff
	real,				intent(in)	:: rho
	real,				intent(in)	:: v1
	integer,			intent(in)	:: nhi
	real,				intent(in)	:: dhi
	integer,			intent(in)	:: ng
	real,				intent(in)	:: dg
	real,dimension(nhi),intent(out)	:: zih
	real,dimension(ng),	intent(out)	:: zig
	real,dimension(nmd),intent(out)	:: zimh
	real,dimension(nmd),intent(out)	:: zimg
	real,dimension(nhi),intent(out)	:: hi
	real,dimension(ng),	intent(out)	:: g
	real,dimension(nmd),intent(out)	:: mzih
	real,dimension(nmd),intent(out)	:: mzig
	integer,			intent(out) :: count_hi
	integer,			intent(out) :: count_g
	integer,			intent(out) :: count_mzih
	integer,			intent(out) :: count_mzig
	
	!! internal variables
	integer :: ihd,imd
	real	:: md, hd
	real	:: s_alpha_s, s_alpha_r, c_alpha_s, c_alpha_r
	real	:: s_beta_s, s_beta_r, c_beta_s, c_beta_r
	real	:: ts1, tr1, ts2, tr2
	real	:: hzi, zzi, mzi, zzig
	real	:: alpha_s, alpha_r, beta_s, beta_r, gamma
	real	:: zdiff
	real	:: tildeZD, tildeZs 
	real	:: den1, num1, vnmo 
	real	:: temp1, temp2, zwb, zwb2, rho2, tm
	real	:: one_cmp_diffmul_dip

	! initialize counters
    count_hi=0;	count_g=0; count_mzih=0; count_mzig=0
	rho2=rho**2
    vnmo=v1/cos(2*dip)
	zdiff=xdiff*tan(dip)

	! loop over data space cmps
    do imd=1,nmd

        md=omd+(imd-1)*dmd
        zwb=md*tan(dip);
        zwb2=zwb*zwb;
       	tildeZD=md*sin(dip);
	
		! loop over data space surface offsets
        do ihd=1,nhd

            hd=ohd+(ihd-1)*dhd;

            ! compute the traveltime of the multiple
            tm=one_cmp_diffmul_dip(dip,md,hd,v1,xdiff);
            tr1=(sqrt((hd+(md-xdiff))**2+zdiff**2))/v1;

            ! compute the perpendicular distance between the source 
			! and the reflector
            tildeZs=tildeZD-hd*sin(dip);

            ! compute the takeoff angles at the source and the receiver
            alpha_r=atan((hd+md-xdiff)/zdiff);
            den1=v1*(tm-tr1);
            num1=2*tildeZs*sin(dip)+(hd+xdiff-md);
            alpha_s=asin(num1/den1)-2*dip;

            ! compute the sines and cosines of the take-off angles (alphas)
			! and the refracted angles (betas) at the source and receiver
            s_alpha_s=sin(alpha_s);
            c_alpha_s=cos(alpha_s);
            s_alpha_r=sin(alpha_r);
            c_alpha_r=cos(alpha_r);
            temp1=1-rho2*(sin(alpha_s+dip))**2;
            if (temp1<0) then
                cycle;
            end if
            s_beta_s=rho*sin(alpha_s+dip)*cos(dip)-sqrt(temp1)*sin(dip);
            c_beta_s=sqrt(temp1)*cos(dip)+rho*sin(alpha_s+dip)*sin(dip);
            temp2=1-rho2*(sin(alpha_r-dip))**2;
            if (temp2<0) then
                cycle;
            end if
            s_beta_r=rho*sin(alpha_r-dip)*cos(dip)+sqrt(temp2)*sin(dip);
            c_beta_r=sqrt(temp2)*cos(dip)-rho*sin(alpha_r-dip)*sin(dip);

            ! compute the individual traveltime segments
            ts1=tildeZs/(v1*cos(dip+alpha_s));
            ts2=(ts1*c_alpha_s)/cos(alpha_s+2*dip);
            tr2=(ts2*cos(alpha_s+dip))/cos(alpha_s+3*dip);

            ! compute the image space coordinates
			call compute_image_coords(hd,v1,ts1,s_alpha_s,tr1,ts2,tr2,s_alpha_r,&
				& rho,c_alpha_r,c_alpha_s,s_beta_s,s_beta_r,hzi,zwb,c_beta_r,&
				& c_beta_s,zzi,md,mzi,gamma,zzig)
			if (gamma==-9999.) cycle

			! set output arrays
			call set_output(mzi,xmzi,dmd,count_hi,hi,hzi,zih,zzi,nhi,xhi,dhi,&
				& count_mzih,zimh,mzih,nmd,count_g,g,gamma,zig,zzig,&
				& ng,xg,dg,count_mzig,mzig,zimg)
        end do
    end do
	return
end subroutine dip_diffmul_cigs_moveout

real function one_cmp_diffmul_dip(dip,xg,hd,v1,xdiff);

	!! argument variables
	real	:: dip
	real	:: xg		!! gather (CMP) location
	real	:: hd		!! half offset
	real	:: v1
	real	:: xdiff

	!! internal variables
	real	:: xs, xr, zs, z0, zdiff
	real	:: xis, xir, zis, xmsdiff, zm1diff, xm1diff 
	real	:: ddiff, betas, den
	real	:: l1,l2,l3,l4

	xs=xg-hd;
	xr=xg+hd;

    ! compute depths of the reflector at the source position
    zs=xs*tan(dip);  ! vertical distance
    z0=zs*cos(dip);  ! minimum distance
    zdiff=xdiff*tan(dip);

    ! compute coordinates of the image source
    xis=xs-2*z0*sin(dip);
    zis=2*z0*cos(dip);

    ! compute multiple bounce at the surface
    xmsdiff=(xis*zdiff+zis*xdiff)/(zdiff+zis);

    ! compute coordinates of first bounce of multiple at the reflector
    den=zis*(xs-z0*sin(dip))-z0*cos(dip)*(xis-xmsdiff);
    zm1diff=(z0*xmsdiff*zis*cos(dip))/den;
    xm1diff=(zm1diff*(xis-xmsdiff)+zis*xmsdiff)/zis;

    ! compute takeoff angle with respect to the vertical
    ddiff=sqrt(zis**2+(xmsdiff-xis)**2);
    betas=asin((xmsdiff-xs)*cos(dip)/ddiff)-dip;

    ! compute traveltime for diffracted multiple
    l1=(zs*cos(dip))/cos(dip+betas);
    l2=(zs*cos(dip)*cos(betas))/(cos(dip+betas)*cos(2*dip+betas));
    l3=(zs*cos(dip)*cos(betas))/(cos(2*dip+betas)*cos(3*dip+betas));
    l4=sqrt((xr-xdiff)**2+zdiff**2);
    one_cmp_diffmul_dip=(l1+l2+l3+l4)/v1;
	return

end function one_cmp_diffmul_dip

subroutine sort_two_arrays(N,array1,array2)
	
	!! subroutine arguments
	integer,			intent(in)		:: N
	real,dimension(N),	intent(inout)	:: array1
	real,dimension(N),	intent(inout)	:: array2

	!! internal variables
	integer,dimension(N)	:: count
	real,dimension(N)		:: temp1,temp2
	integer					:: i,j

	! initialize auxiliary arrays
	count=0; temp1=array1; temp2=array2

	! compute the table of indices that give the right position
	! of each entry of array1
	do i=N,2,-1
		do j=i-1,1,-1

			if (array1(i)<array1(j)) then
				count(j)=count(j)+1
			else
				count(i)=count(i)+1
			end if
		end do
	end do
		
	do i=1,N
		array1(count(i)+1)=temp1(i)
		array2(count(i)+1)=temp2(i)
	end do
	!do i=1,N
	!write(0,*)'array1=',array1(i),'temp1=',temp1(i),'count=',count(i)+1
	!end do
	!write(0,*)' '

end subroutine  sort_two_arrays
		

subroutine set_output(mzi,xmzi,dmd,count_hi,hi,hzi,zih,zzi,nhi,xhi,dhi,&
	& count_mzih,zimh,mzih,nmd,count_g,g,gamma,zig,zzig,ng,xg,dg,&
	& count_mzig,mzig,zimg)

	!! subroutine arguments
	real,				intent(in)	:: mzi
	real,				intent(in)	:: hzi
	real,				intent(in)	:: zzi
	real,				intent(in)	:: xmzi
	real,				intent(in)	:: xhi
	integer,			intent(in)	:: nhi
	real,				intent(in)	:: dhi
	real,				intent(in)	:: gamma
	real,				intent(in)	:: zzig
	real,				intent(in)	:: xg
	integer,			intent(in)	:: ng
	real,				intent(in)	:: dg
	integer,			intent(in)	:: nmd
	real,				intent(in)	:: dmd
	real,dimension(nhi),intent(out)	:: zih
	real,dimension(ng),	intent(out)	:: zig
	real,dimension(nmd),intent(out)	:: zimh
	real,dimension(nmd),intent(out)	:: zimg
	real,dimension(nhi),intent(out)	:: hi
	real,dimension(ng),	intent(out)	:: g
	real,dimension(nmd),intent(out)	:: mzih
	real,dimension(nmd),intent(out)	:: mzig
	integer,			intent(inout) :: count_hi
	integer,			intent(inout) :: count_g
	integer,			intent(inout) :: count_mzih
	integer,			intent(inout) :: count_mzig

	! set output arrays
    if (abs(mzi-xmzi)<=dmd/2) then
		count_hi=count_hi+1;
		hi(count_hi)=hzi/2
		zih(count_hi)=zzi
		if(count_hi==nhi) return;
	end if
	if (abs(hzi-xhi)<=dhi/2) then
		count_mzih=count_mzih+1;
		mzih(count_mzih)=mzi
		zimh(count_mzih)=zzi
		if(count_mzih==nmd) return;
	end if
	if (abs(mzi-xmzi)<=dmd/2) then
		count_g=count_g+1;
		g(count_g)=gamma
		zig(count_g)=zzig
		if(count_g==ng) return;
	end if
	if (abs(gamma-xg)<=dg/2) then
		count_mzig=count_mzig+1;
		mzig(count_mzig)=mzi
		zimg(count_mzig)=zzi
		if(count_mzig==nmd) return;
	end if

end subroutine set_output 

subroutine compute_image_coords(hd,v1,ts1,s_alpha_s,tr1,ts2,tr2,&
	& s_alpha_r,rho,c_alpha_r,c_alpha_s,s_beta_s,s_beta_r,hzi,&
	& zwb,c_beta_r,c_beta_s,zzi,md,mzi,gamma,zzig)

	!! subroutine arguments
	real,	intent(in)	:: hd
	real,	intent(in)	:: v1
	real,	intent(in)	:: ts1
	real,	intent(in)	:: ts2
	real,	intent(in)	:: tr1
	real,	intent(in)	:: tr2
	real,	intent(in)	:: s_alpha_s
	real,	intent(in)	:: s_alpha_r
	real,	intent(in)	:: c_alpha_r
	real,	intent(in)	:: c_alpha_s
	real,	intent(in)	:: rho
	real,	intent(in)	:: s_beta_s
	real,	intent(in)	:: s_beta_r
	real,	intent(in)	:: c_beta_s
	real,	intent(in)	:: c_beta_r
	real,	intent(in)	:: zwb
	real,	intent(in)	:: md
	real,	intent(out)	:: hzi
	real,	intent(out)	:: zzi
	real,	intent(out)	:: mzi
	real,	intent(out)	:: gamma
	real,	intent(out)	:: zzig

	!! internal variables
	real				:: beta_s,beta_r
	real				:: tts2,ttr2

	!! compute refracted angles
	beta_s=asin(s_beta_s)
	beta_r=asin(s_beta_r)
	
	! compute the traveltime of the refracted ray segments
	tts2=(tr1*c_alpha_r-ts1*c_alpha_s+rho*(ts2+tr2)*c_beta_r)/&
		& (rho*(c_beta_s+c_beta_r))
	ttr2=(ts1*c_alpha_s-tr1*c_alpha_r+rho*(ts2+tr2)*c_beta_s)/&
		& (rho*(c_beta_s+c_beta_r))

	! compute the image space coordinates
	hzi=2*hd-v1*(ts1*s_alpha_s+tr1*s_alpha_r+ &
		& rho*(tts2*s_beta_s+ttr2*s_beta_r))
	zzi=v1*(ts1*c_alpha_s+rho*tts2*c_beta_s)
	mzi=md+v1/2*(ts1*s_alpha_s-tr1*s_alpha_r+ &
		& rho*(tts2*s_beta_s-ttr2*s_beta_r))

	! compute aperture angle and depth of the image point in ADCIGs
	gamma=0.5*(beta_s+beta_r)
	if (abs(gamma-pi/2)<0.1) then
		gamma=-9999.
	else
		zzig=zzi-0.5*hzi*tan(gamma)
	end if

end subroutine compute_image_coords
end module image_moveouts
